# Copyright 1999 by Marco d'Itri <md@linux.it>
# You can use this software under the terms of the GPL. If we meet some day,
# and you think this stuff is worth it, you can buy me a beer in return.

package News::Nocem;

use strict;
use vars qw($VERSION);

$VERSION="0.1";

sub new {
	my ($class, $issuer, $prefix)=@_;
	@_ >= 2 or die 'usage: News::Nocem->new(ISSUER [, IDPREFIX])';

	my $self={
		count => 0, type => 'spam', action => 'hide', id => [], ng => [],
	};
	bless $self, $class;

	my ($sec,$min,$hr,$mday,$mon,$year) = gmtime(time);
	$self->{'issuer'}=$issuer if (defined $issuer);
	if (defined $prefix) {
		$self->{'noticeid'}=sprintf('%s%04d%02d%02d%02d%02d%02d.%04x',
				$prefix, $year+1900, $mon, $mday, $hr, $min, $sec,
				0xFFFF & (rand(32768) ^ $$));
	}
	return $self;
}

sub add {
	my ($self, $id, $ng)=@_;
	@_ == 3 or die 'usage: News::Nocem->add(Message-Id, Newsgroups)';

	$ng =~ s/\s*,\s*/ /g;
	push @{$self->{id}}, $id;
	push @{$self->{ng}}, $ng;
}

sub print {
	my $self=shift;

	die 'Notice-ID: field not defined' if (not defined $self->{'noticeid'});
	return undef if (scalar @{$self->{id}}==0);
#	use Data::Dumper; print Data::Dumper->Dump([\$self], ["*self"]);
	my @r;
	push @r, '@@BEGIN NCM HEADERS';
	push @r, 'Version: 0.9';
	push @r, 'Issuer: ' . $self->{'issuer'};
	push @r, 'Type: ' . $self->{'type'};
	push @r, 'Action: ' . $self->{'action'};
	push @r, 'Notice-ID: ' . $self->{'noticeid'};
	push @r, 'Count: ' . scalar @{$self->{id}};
	push @r, '@@BEGIN NCM BODY';
	for (my $i=0; $i < scalar @{$self->{id}}; $i++) {
		push @r, $self->{id}[$i] . "\t"
			. join("\n\t", split(/\s+/, $self->{ng}[$i]));
	}
	push @r, '@@END NCM BODY';
	@r;
}

sub type {
	my $self=shift;
	$self->{'type'}=shift if @_;
	$self->{'type'};
}

sub action {
	my $self=shift;
	$self->{'action'}=shift if @_;
	$self->{'action'};
}

sub noticeid {
	my $self=shift;
	$self->{'noticeid'}=shift if @_;
	$self->{'noticeid'};
}

sub issuer { $_[0]->{'issuer'} }
sub count { $_[0]->{'count'} }

__END__

=head1 NAME

News::Nocem - generate a NoCeM notice

=head1 SYNOPSIS

	use News::Nocem;
	my $n=News::Nocem->new('nocem@news.nic.it', 'gcnnocem.');
	$n->add('<id@host>', 'one.newsgroup, another.newsgroup, another.else');
	$n->add('<id@otherhost>', 'alt.spammed.newsgroup');
	print join "\n", ($n->print, '');

=head1 DESCRIPTION

This module can generate a NoCeM notice (only the body of the article).

=head1 CONSTRUCTOR

=over 4

=item new ( ISSUER [, IDPREFIX] )

This is the constructor for a new News::Nocem object. C<ISSUER> is the
content of the I<Issuer:> field, C<IDPREFIX> is an optional string
prefixed to the automatically generated I<Notice-ID:> field.
If no string is provided the field is not generated.

=back

=head1 METHODS

=over 4

=item add ( Message-Id, Newsgroups )

Add a new article to the notice. The two parameters are its Message-ID
and the newsgroups where it is posted. The newsgroups can be separated
by any sequence of commas and white space.

=item print ()

Returns the notice as a list of lines.

=item others

The type, action and noticeid methods can be used to read or set the
corresponding fields.

The issuer and count methods can be used to read the corresponding
fields.

=back

=head1 AUTHOR

Marco d'Itri E<lt>md@linux.itE<gt>

=head1 LICENSE

You can use this software under the terms of the GPL. If we meet some day,
and you think this stuff is worth it, you can buy me a beer in return.

=cut

